# Needs  Neopixel strip on GP15
# Modified from Official Rasp Pi example here:
# https://github.com/micropython/micropython/tree/master/examples/bluetooth
# Tony Goodhew 23 June 2023

import bluetooth
import random
import struct
import time
from ble_advertising import advertising_payload
from micropython import const

# Neopixel additional material ############
from machine import Pin
from neopixel import NeoPixel
strip = NeoPixel(Pin(15), 8)

#   End of LED additional material  #########

_IRQ_CENTRAL_CONNECT = const(1)
_IRQ_CENTRAL_DISCONNECT = const(2)
_IRQ_GATTS_WRITE = const(3)

_FLAG_READ = const(0x0002)
_FLAG_WRITE_NO_RESPONSE = const(0x0004)
_FLAG_WRITE = const(0x0008)
_FLAG_NOTIFY = const(0x0010)

_UART_UUID = bluetooth.UUID("6E400001-B5A3-F393-E0A9-E50E24DCCA9E")
_UART_TX = (
    bluetooth.UUID("6E400003-B5A3-F393-E0A9-E50E24DCCA9E"),
    _FLAG_READ | _FLAG_NOTIFY,
)
_UART_RX = (
    bluetooth.UUID("6E400002-B5A3-F393-E0A9-E50E24DCCA9E"),
    _FLAG_WRITE | _FLAG_WRITE_NO_RESPONSE,
)
_UART_SERVICE = (
    _UART_UUID,
    (_UART_TX, _UART_RX),
)


class BLESimplePeripheral:
    def __init__(self, ble, name="mpy-uart"):
        self._ble = ble
        self._ble.active(True)
        self._ble.irq(self._irq)
        ((self._handle_tx, self._handle_rx),) = self._ble.gatts_register_services((_UART_SERVICE,))
        self._connections = set()
        self._write_callback = None
        self._payload = advertising_payload(name=name, services=[_UART_UUID])
        self._advertise()

    def _irq(self, event, data):
        # Track connections so we can send notifications.
        if event == _IRQ_CENTRAL_CONNECT:
            conn_handle, _, _ = data
            print("New connection", conn_handle)
            self._connections.add(conn_handle)
        elif event == _IRQ_CENTRAL_DISCONNECT:
            conn_handle, _, _ = data
            print("Disconnected", conn_handle)
            self._connections.remove(conn_handle)
            # Start advertising again to allow a new connection.
            self._advertise()
        elif event == _IRQ_GATTS_WRITE:
            conn_handle, value_handle = data
            value = self._ble.gatts_read(value_handle)
            if value_handle == self._handle_rx and self._write_callback:
                self._write_callback(value)

    def send(self, data):
        for conn_handle in self._connections:
            self._ble.gatts_notify(conn_handle, self._handle_tx, data)

    def is_connected(self):
        return len(self._connections) > 0

    def _advertise(self, interval_us=500000):
        print("Starting advertising")
        self._ble.gap_advertise(interval_us, adv_data=self._payload)

    def on_write(self, callback):
        self._write_callback = callback

# This is the MAIN LOOP
def demo():    # This part modified to control Neopixel strip
    ble = bluetooth.BLE()
    p = BLESimplePeripheral(ble)

    def on_rx(v):  # v is what has been received
        ss = str(v)     # Unpack RGB values
        l = len(ss)
        ss = ss[2:l-1]     # Strip to essentials  
        p = ss.find(",")   # Find first comma
        r = int(ss[0:p])   # Extract RED value
        ss = ss[p+1:]      # Remove red part
        p = ss.find(",")   # Find comma separator
        g = int(ss[0:p])   # Extract GREEN value
        b = int(ss[p+1:])  # Extract BLUE value
        print(r,g,b)       # Print RGB values

        # Set the RGB colours
        strip[0] = (r,g,b) # Mixed pixel
        
        strip[7] = [0,0,b] # individual components
        strip[6] = [0,g,0]
        strip[5] = [r,0,0]
        # Send the data to the strip
        strip.write()
        
    p.on_write(on_rx)


if __name__ == "__main__":
    demo()